'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _fs = require('fs');

var _express = require('express');

var _express2 = _interopRequireDefault(_express);

var _cors = require('cors');

var _cors2 = _interopRequireDefault(_cors);

var _multer = require('multer');

var _multer2 = _interopRequireDefault(_multer);

var _bodyParser = require('body-parser');

var _fileService = require('./file-service');

var _log = require('./log');

var _log2 = _interopRequireDefault(_log);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-disable no-underscore-dangle */
var saveFile = function saveFile(request, response, filename) {
  _log2.default.verbose('Saving file', request.file, filename);
  var result = (0, _fileService.writeFile)(request.file);
  if (request.query._postmessage) {
    if (request.query._postmessageid) {
      result.data._postmessageid = request.query._postmessageid;
    }
    response.status(result.status).header('Content-Type', 'text/html').send('<!DOCTYPE html><script>parent.postMessage(JSON.stringify(' + JSON.stringify(result.data) + '), \'*\');</script>');
  } else {
    response.status(result.status).send(result.data);
  }
};
/* eslint-enable no-underscore-dangle  */

exports.default = {
  init: function init(options) {
    var storage = void 0;
    if (options.storage.type === 'disk') {
      if (options.storage.path && !(0, _fs.existsSync)(options.storage.path)) {
        _log2.default.error(options.storage.path, 'does not exist');
        throw new Error(options.storage.path + ' does not exist');
      }

      storage = (0, _multer.diskStorage)({
        destination: function destination(req, file, cb) {
          cb(null, options.storage.path || '/tmp');
        },
        filename: function filename(req, file, cb) {
          cb(null, file.originalname || file.fieldname);
        }
      });
    } else {
      storage = (0, _multer.memoryStorage)();
    }
    var upload = (0, _multer2.default)({ storage: storage });

    var app = (0, _express2.default)();
    app.use((0, _bodyParser.urlencoded)({
      extended: true
    }));
    app.use((0, _bodyParser.json)());
    app.use((0, _cors2.default)());

    app.get('/' + options.route + '/:filename/size', function (request, response) {
      var result = (0, _fileService.getFileSize)(request.params.filename);
      response.status(result.status).send(result.data);
    });

    app.get('/' + options.route + '/:filename', function (request, response) {
      var result = (0, _fileService.readFile)(request.params.filename);
      response.status(result.status).send(result.data);
    });
    app.delete('/' + options.route + '/:filename', function (request, response) {
      var result = (0, _fileService.removeFile)(request.params.filename);
      response.status(result.status);
    });

    app.post('/' + options.route, upload.single('file'), function (request, response) {
      saveFile(request, response, request.file.originalname);
    });

    app.post('/' + options.route + '/:filename', upload.single('file'), function (request, response) {
      saveFile(request, response, request.params.filename);
    });

    app.post('/' + options.route + '/chunk/:filename', upload.single('file'), function (request, response) {
      var result = (0, _fileService.writeFileChunk)(request.params.filename, request.file.buffer, request.body[options.chunkNumber || 'chunknumber']);
      response.status(result.status).send(result.data);
    });

    app.post('/' + options.route + '/assemble/:filename', function (request, response) {
      var result = (0, _fileService.assembleFileChunks)(request.params.filename, request.body[options.totalSize || 'totalsize']);
      response.status(result.status).send(result.data);
    });

    return app;
  },
  run: function run(options) {
    var server = this.init(options);
    server.listen(options.port, function () {
      _log2.default.info('Server ready. Configuration:');
      _log2.default.info('  * Storage: %s %s', options.storage.type, options.storage.type === 'disk' && options.storage.path || '');
      _log2.default.info('  * Port:', options.port);
      _log2.default.info('  * Routes:', '/' + options.route);
      _log2.default.info('  * Verbose:', options.verbose ? 'yes' : 'no');
    });
  }
};